import typing as tp
import sys
import time
import random

import requests
from selenium import webdriver
from selenium.webdriver.chrome.options import Options
from selenium.webdriver.chrome.service import Service
from webdriver_manager.chrome import ChromeDriverManager

from conqueror.msg_generator.messages import generate_message_client

driver = None
url = None
zalogowano = False
akceptuje_off = False


def spam(login, haslo, url):
    global driver, zalogowano, akceptuje_off

    try:
        print("Spam dla URL:", url)
        driver.get(url)

        # klikamy ciasteczka
        if not akceptuje_off:
            time.sleep(2)
            try:
                a = driver.find_elements_by_xpath("//*[contains(text(), 'Akceptuj')]")
                for i in a:
                    i.click()
                akceptuje_off = True
            except Exception as ex2:
                print(ex2)

        # klikamy recenzje
        driver.execute_script("window.scrollTo(0, 600)")
        driver.execute_script("window.scrollTo(0, 600)")
        t1x = "_name_reviews"
        t1 = driver.find_element_by_class_name(t1x)
        t1.click()
        print("Kliknieto przycisk Recenzje")

        # klikamy 5 gwiazdek
        time.sleep(2)
        t3 = driver.find_elements_by_class_name("business-rating-edit-view__star")
        t3[-1].click()
        print("Kliknieto 5 gwiazdek")
        time.sleep(1)
    
        if not zalogowano:
            # logujemy sie
            driver.find_element_by_class_name("login-dialog-view__button").click()

            time.sleep(1)

            # login
            login_tb = driver.find_element_by_id('passp-field-login')
            login_tb.send_keys(login)

            # przycisk logowania
            logib_btn = driver.find_element_by_id('passp:sign-in')
            logib_btn.click()

            time.sleep(1)

            # haslo
            login_tb = driver.find_element_by_id('passp-field-passwd')
            login_tb.send_keys(haslo)

            time.sleep(0.7)

            # przycisk logowania 2
            driver.find_element_by_class_name("Button2_type_submit").click()

            time.sleep(1)

            # przy pierwszym logowaniu ever chce telefon
            try:
                driver.find_element_by_class_name("Button2_view_pseudo").click()
                time.sleep(1)
            except Exception as ex3:
                print(ex3)

            driver.execute_script("window.scrollTo(0, 600)")

            time.sleep(1)

            # klikemy 5 gwiazdek
            t2x = "business-rating-edit-view"
            t2 = driver.find_element_by_class_name(t2x)
            t3 = t2.find_elements_by_class_name("business-rating-edit-view__star")
            t3[-1].click()

            zalogowano = True
            print("Zalogowano")

        # losujemy wiadomosc
        msg = generate_message_client()
        print("Wylosowany komunikat:", msg)

        # wpisujemy wiadomosc
        t4 = driver.find_element_by_class_name("textarea__control")
        print("Zlokalizowano element textarea dla wiadomosci")
        t4.send_keys(msg)
        print("Wpisano wiadomosc")

        # wysylamy recenzje
        driver.execute_script("window.scrollTo(0, 300)")
        time.sleep(1)
        driver.find_element_by_class_name("business-review-form__controls").find_elements_by_tag_name("div")[0].click()

        print('OK for url: ', url)
        try:
            requests.get(f'https://yandex.henrietta.com.pl/v1/add-review/{login}')
        except (requests.RequestException, IOError, OSError):
            print('Could not update the coordinator!')
        time.sleep(0.5)
    except Exception as ex:
        print(ex)


def get_target_list() -> tp.List[str]:
    cities = [city['name'] for city in requests.get('https://yandex.henrietta.com.pl/v1/view-cities').json()]

    chosen_cities = random.sample(cities, 10)

    for city in chosen_cities:
        resp = requests.get(f'https://yandex.henrietta.com.pl/v1/view-businesses/{city}')
        resp.raise_for_status()
        j = resp.json()
        print("RESP:", j)
        target_list.extend(j)

    random.shuffle(target_list)
    return target_list


if __name__ == '__main__':

    if len(sys.argv) < 3:
        print('''The correct way to load this script is to'
python -m conqueror <login to yandex> <password to yandex>
''')
        sys.exit(1)

    target_list = []

    options = Options()
    options = webdriver.ChromeOptions()
    if '--headless' in sys.argv:
        print('The software will not function correctly in headless mode, ignoring the command')
    driver = webdriver.Chrome(service=Service(ChromeDriverManager().install()), options=options)

    print("Pomieszano cele")
    login, haslo = sys.argv[1:3]
    print('Login=', login, 'password=', haslo)

    try:
        while True:
            for i in get_target_list()[:100]:
                print("Cel:", i)
                spam(login, haslo, f'https://yandex.ru/maps/org/itle/{i}')
    except KeyboardInterrupt:
        print('Aborting')
    driver.close()




import typing as tp

import requests

from .yandex_api_schema import YandexAPISchema


def search_on_yandex(**kwargs: tp.Dict[str, str]):
    """Search for places | business.

    :returns: Json with scraped information about points.
    """
    url: str = "https://search-maps.yandex.ru/v1/?"
    resp = requests.get(url=url, timeout=5, params={**kwargs})
    resp.raise_for_status()
    return resp.json()


def extract_point_coordinates(scraped_points) -> None:
    """If it's point, extract coordinates(latitude & longitude)."""
    try:
        for feature_key in scraped_points["features"]:
            for geo_key, coordinates in feature_key["geometry"].items():
                if isinstance(coordinates, list):
                    latitude, longitude = coordinates[0], coordinates[1]
                    print(f"Latitude: {latitude}, Longitude: {longitude}")
    except KeyError as k_err:
        raise k_err



import typing as tp
import dataclasses
import os
import json
import pathlib
import typing as tp
from collections import defaultdict

BASE_FILES_LOC: str = os.path.join(
    pathlib.Path(__file__).parent.parent.parent, "resources"
)


@dataclasses.dataclass
class City:
    city_in_english: str
    city_in_russian: str
    population: float
    radius_km: float
    lat: float
    lon: float


def read_cities() -> tp.List[City]:
    with open(os.path.join(BASE_FILES_LOC, "russian-cities.json")) as json_f:
        data = json.load(json_f)
        results = []
        columns = list(data.keys())
        entries = [key for key in data[columns[0]]]

        for key in entries:

            dct = {}
            for column in columns:
                dct[column] = data[column][key]

            results.append(City(**dct))
    return results



from conqueror.endpoints import add_account
from conqueror.endpoints import add_business
from conqueror.endpoints import view_business
from conqueror.endpoints import view_messages
from conqueror.endpoints import view_cities
from conqueror.endpoints import add_review



import math


def degrees_to_kilometers_lat(degrees: int) -> float:
    """
    Convert degrees of longitude into kilometres
    :param degrees:
    :return:
    """

    return (63567523 * degrees / 90) / 1000


def degrees_to_kilometers_lon(degrees: float, degrees_lat: float) -> float:
    return degrees * 111.320 * math.cos(degrees_lat)


def kilometers_to_degrees_lon(kilometres: float, degrees_lat: float) -> float:
    return kilometres / 111.320 / math.cos(degrees_lat)


def kilometers_to_degrees_lat(kilometres: float) -> float:
    """
    Convert kilometres of longitude into degrees

    :param degrees:
    :return:
    """

    return kilometres * 90 * 1000 / 63567523



import sys

from satella.instrumentation import Traceback

from conqueror.cassandra import session
from conqueror.scraper.get_object_types import get_random_object_type
from conqueror.scraper.orders_obtainer import OrdersObtainer
from conqueror.scraper.scrap_yandex import search_on_yandex
from conqueror.scraper.yandex_api_schema import YandexAPISchema


def run():
    po = OrdersObtainer()
    for i in range(0, int(sys.argv[2])):
        city = po.get_random_city()
        point = po.get_random_point(city)

        utility_point = get_random_object_type()

        query = YandexAPISchema(apikey=sys.argv[1],
                                text=f'{city.city_in_russian} {utility_point}',
                                ll=f'{point[1]},{point[0]}')

        query_d = dict(query)
        points = search_on_yandex(**query_d)

        try:
            for data_point in points['features']:

                if 'CompanyMetaData' not in data_point['properties']:
                    continue
                name = data_point['properties']['name']
                company_id = data_point['properties']['CompanyMetaData']['id']
                geo_lon, geo_lat = data_point['geometry']['coordinates']
                session.execute(
                    'INSERT INTO businesses (sector, businessId, geoLat, geoLon, name) VALUES (%s, %s, %s, %s, %s)',
                    (
                        city.city_in_english,
                        company_id, geo_lat, geo_lon, name
                    ))
        except Exception as e:
            print(Traceback().pretty_format())
        print('Inserted ', len(points['features']), ' data points into Cassandra')



from conqueror.app import app
from conqueror.cassandra import session
from conqueror.utils import as_json


@app.route('/v1/view-businesses/<string:sector>')
@as_json
def get_businesses(sector: str):
    """
    Get a list of Russian businesses to post reviews for
    ---
    parameters:
    - in: path
      name: sector
      schema:
        type: string
      description: Name of Russian city in Latin alphabet
      required: true
    ---
    responses:
        '200':
            description: OK
            content:
                application/json:
                    schema:
                        type: array
                        items:
                            type: string
                            description: Business ID of the object
    """
    resp = session.execute('SELECT businessId FROM businesses WHERE sector=%s', (sector,))
    results = []
    for datum in resp:
        results.append(datum[0])
    return results


@app.route('/v1/view-businesses')
@as_json
def get_businesses_all():
    """
    Get a list of all Russian businesses to post reviews for
    ---
    responses:
        '200':
            description: OK
            content:
                application/json:
                    schema:
                        type: array
                        items:
                            type: string
                            description: Business ID of the object
    """
    resp = session.execute('SELECT businessId FROM businesses')
    results = []
    for datum in resp:
        results.append(datum[0])
    return results



import os

from cassandra.cluster import Cluster

keyspace = os.environ.get('CASSANDRA_KEYSPACE', 'yandex')

cluster = Cluster([os.environ.get('CASSANDRA_HOSTNAME', 'cassandra')])
session = cluster.connect(keyspace)



"""Schema for Yandex places API."""

import dataclasses
import typing as tp


@dataclasses.dataclass
class YandexAPISchema:
    """Provide correct schema to talk with yandex API.

    :param text: Search query, by cit name, business name and so on.
    :param lang: Response language.
    :param apikey: Your api key to yandex places API.
    :param ll: Search area center.
    :param spn: Search area size.
    :param type: Object types:
        - 'geo', toponyms
        - 'biz, businesses
        - 'omitted', automatically detect the type based on the query text
    """
    text: str
    lang: str = "ru_ru"
    apikey: str = "558d43d5-79eb-40c6-938d-e303ab670a3b"
    ll: tp.Optional[str] = ""
    spn: tp.Optional[str] = "0.02,0.02"
    type: tp.Optional[str] = ""
    results: tp.Optional[str] = "500"

    def __iter__(self):
        """Allow to transform dataclass into dict."""
        yield "text", self.text
        yield "lang", self.lang
        yield "apikey", self.apikey
        yield "ll", self.ll
        yield "spn", self.spn
        yield "results", self.results



from flask import request

from conqueror.app import app
from conqueror.cassandra import session
from conqueror.utils import as_json


@app.route('/v1/add-businesses', methods=['PUT'])
@as_json
def add_business():
    """
    Adds a business
    ---
    requestBody:
        content:
            application/json:
                schema:
                    type: array
                    items:
                        type: object
                        properties:
                            businessId:
                                type: string
                                description: Business ID
                            sector:
                                type: string
                                description: Business' sector
                            lat:
                                type: float
                                description: Geographical latitude in degrees. + is North
                            lon:
                                type: float
                                description: Geographical longitude in degrees. + is East
                            name:
                                type: string
                                description: Optional object name
                        required:
                            - businessId
                            - sector
    responses:
        '201':
            description: Business added
    """
    data = request.get_json()

    for datum in data:
        businessId = datum['businessId']
        sector = datum['sector']
        lat = datum.get('lat')
        lon = datum.get('lon')
        name = datum.get('name')

        session.execute('INSERT INTO businesses (businessId, sector, GeoLat, geoLon, name) VALUES (%s, %s, %s, %s, %s)',
                        (businessId, sector, lat, lon, name))
        session.execute('INSERT INTO sectors (sector) VALUES (%s)', (sector,))
    return {}, 201



from conqueror.app import app
from conqueror.scraper.data_reader import read_cities
from conqueror.utils import as_json


@app.route('/v1/view-cities')
@as_json
def get_cities():
    """
    Get a list of Russian cities
    ---
    parameters:
    - in: path
      name: sector
      schema:
        type: string
      description: Name of Russian city in Latin alphabet
      required: true
    ---
    responses:
        '200':
            description: OK
            content:
                application/json:
                    schema:
                        type: array
                        items:
                            type: object
                            properties:
                                name:
                                    type: string
                                    description: Name of the city in Latin
                                population:
                                    type: integer
                                    description: Population of the city
                            required:
                                - name
                                - population
    """
    cities = read_cities()
    results = []
    for city in cities:
        results.append({'name': city.city_in_english,
                        'population': city.population})
    return results




import math
import queue
import threading
from random import random
import time
from satella.coding.concurrent import TerminableThread

from conqueror.scraper.orders_obtainer import OrdersObtainer

API_ENQUIRY_EACH_SECOND = 4  # a query each 4 seconds


def get_fai(lat: float, lon: float, radius: float) -> tuple[float, float]:
    """
    Get a coordinates for a fairly chosen fircle within lat and lon 
    
    :param lat: geo latitude in degrees, + is N
    :param lon: geo longitude in degrees, + is E
    :param radius: radius in kilometres 
    :return: a tuple of (lat, lon)
    """
    radius = 2 * math.pi * random()
    u = random() + random()
    if u > 1:
        r = 2 - u
    else:
        r = u
    return [r * math.cos(t), r * math.sin(t)]


class PermissionGenerator(threading.Thread):
    def __init__(self):
        super().__init__(daemon=True)
        self.queue = queue.Queue()

    def run(self):
        while True:
            self.queue.put(1)
            time.sleep(API_ENQUIRY_EACH_SECOND)


class ScrapingThread(TerminableThread):
    def __init__(self, pg: PermissionGenerator):
        super().__init__()
        self.pg = pg

    def run(self):
        while not self._terminating:
            self.pg.queue.get(True)  # obtain the permission to scrape



from flask import Response
from satella.json import json_encode
from satella.coding import wraps
from .geom2d import kilometers_to_degrees_lon, kilometers_to_degrees_lat, \
    degrees_to_kilometers_lon, degrees_to_kilometers_lat


def as_json(fun):
    """
    Functions decorated with this are expect to return one of three positions:

    * None - HTTP 204 will be sent
    * tuple/2 - tuple[0] will contain the response and tuple[1] will contain the status code
    * other things - HTTP 200 will be returned

    Every dict and list returned will be stringified with Satella's stringify
    """

    @wraps(fun)
    def outer(*args, **kwargs):
        data = fun(*args, **kwargs)
        if data is None:
            status_code, content_type = 204, None
        else:
            content_type = 'application/json'
            if isinstance(data, tuple):
                if len(data) == 2:
                    data, status_code = data
            else:
                status_code = 200
            data = json_encode(data)

        return Response(data, status=status_code, content_type=content_type)

    return outer



import random

from conqueror.scraper.data_reader import City, read_cities
from conqueror.utils.geom2d import degrees_to_kilometers_lat, degrees_to_kilometers_lon, kilometers_to_degrees_lon, \
    kilometers_to_degrees_lat
from conqueror.utils.point_obtain import random_point


class OrdersObtainer:
    """
    A generator of fair orders
    """

    def __init__(self):
        cities = read_cities()
        self.thresholds = []
        self.cities = []
        pop_cntr = 0
        for i, city in enumerate(cities):
            pop_cntr += city.population
            self.thresholds.append(pop_cntr)
            self.cities.append(city)

    def get_random_city(self) -> City:
        """
        Select a random city thanks to a built-in property distributor
        """
        pop = random.randint(0, self.thresholds[-1])
        for threshold, city in zip(self.thresholds, self.cities):
            if pop < threshold:
                return city
        return city

    def get_random_point(self, city: City) -> tuple[float, float]:
        geoCenterLat, geoCenterLon = city.lat, city.lon

        radis_x, radis_y = degrees_to_kilometers_lat(geoCenterLat), degrees_to_kilometers_lon(geoCenterLon,
                                                                                              geoCenterLat),
        dx, dy = random_point(city.radius_km)
        radis_x += dx
        radis_y += dy
        return kilometers_to_degrees_lat(radis_x), kilometers_to_degrees_lon(radis_y, geoCenterLat)



import requests
from satella.coding.decorators import retry


@retry(3, exc_classes=requests.RequestException)
def get_yandex_request(url, arguments) -> dict:
    """
    Return a JSON object querying Yandex at provided parameters.

    Handling CSRF will be done automatically.

    :param url: URL to ask
    :param arguments: dictionary of arguments to add
    :return: object returned via endpoint
    """
    resp = requests.get(url, params=arguments)
    resp.raise_for_status()
    data = resp.json()
    if list(data.keys()) == ['csrfToken']:
        arguments['csrfToken'] = data['csrfToken']
        return get_yandex_request(url, arguments)
    else:
        return data



import os
import pathlib

import pandas as pd

__all__ = ("DataImporter",)

BASE_FILES_LOC: str = os.path.join(
    pathlib.Path(__file__).parent.parent.parent, "resources"
)


class DataImporter:
    def read_xlsx(self) -> pd.DataFrame:
        """Read data from xlsx file and load the into data frame."""
        xlsx_location_: str = os.path.join(BASE_FILES_LOC, "russian-cities.xlsx")
        raw_df = pd.read_excel(xlsx_location_, engine="openpyxl")
        return raw_df

    def apply_transformations(self, raw_df: pd.DataFrame) -> None:
        """Make columns lower case & snake_case."""
        raw_df.columns = map(str.lower, raw_df.columns)
        df = raw_df.rename(
            columns={
                "city in english": "city_in_english",
                "city in russian": "city_in_russian",
                "radius [km]": "radius_km",
            },
            errors="raise",
        )
        self.save_into_json(df)

    @staticmethod
    def save_into_json(df: pd.DataFrame) -> None:
        """Save prepared data frame into json format."""
        df.to_json("here.json")


d = DataImporter()
raw_df = d.read_xlsx()
d.apply_transformations(raw_df)



from conqueror.app import app
from conqueror.cassandra import session
from conqueror.utils import as_json


@app.route('/v1/add-review/<string:login>')
@as_json
def add_review(login: str):
    """
    Mark a review as added
    ---
    parameters:
    - in: path
      name: login
      schema:
        type: string
      description: Login used to add reviews at
      required: true
    ---
    responses:
        '200':
            description: Added
    """
    resp = list(session.execute('SELECT count FROM review_count WHERE apiKey=%s', (login,)))
    if not resp:
        session.execute('INSERT INTO review_count (apiKey, count) VALUES (%s, 1)', (login, ))
    else:
        entries = resp[0][0]
        session.execute('INSERT INTO review_count (apiKey, count) VALUES (%s, %s)', (login, entries+1))


@app.route('/v1/view-reviews')
@as_json
def view_reviews():
    """
    Get a sum of all reviews added up to this point
    ---
    responses:
        '200':
            description: OK
            content:
                application/json:
                    schema:
                        type: object
                        properties:
                            entries:
                                type: integer
                                description: All entries added
    """
    resp = session.execute('SELECT count FROM review_count')
    entries = sum(d[0] for d in resp)
    return {'entries': entries}



from flask import request

from conqueror.app import app
from conqueror.cassandra import session
from conqueror.utils import as_json


@app.route('/v1/add-yandex-account', methods=['PUT'])
@as_json
def add_yandex_account():
    """
    Add a Yandex account
    ---
    requestBody:
        content:
            application/json:
                schema:
                    type: object
                    properties:
                        email:
                            type: string
                            format: email
                            description: User's email
                        password:
                            type: string
                            format: password
                            description: User's password
    responses:
        '201':
            description: Account added
    """
    data = request.get_json()
    email = data['email']
    password = data['password']

    session.execute('INSERT INTO accounts (email, password, status) VALUES (%s, %s, %s)',
                    (email, password, 0))
    return {}, 201



import math
from random import random


def random_point(r: float) -> tuple[float, float]:
    """
    Pick a mathematically sane random circle around the point of (0, 0) with a radius of 1.

    :param r: radius
    :return: a tuple with either X and Y of the point
    """
    t = 2 * math.pi * random()
    u = random() + random()
    v = 2 - u if u > 1 else u

    return [v * math.cos(t) * r, v * math.sin(t) * r]



from conqueror.app import app
from conqueror.msg_generator import get_messages
from conqueror.utils import as_json


@app.route('/v1/view-messages')
@as_json
def list_messages():
    """
    Return a list of messages to be sent to the Russians
    ---
    responses:
        '200':
            description: OK
            content:
                application/json:
                    schema:
                        type: array
                        items:
                            type: object
                            properties:
                                type:
                                    type: string
                                    description: 1-A
                                content:
                                    type: string
                                    example: [Russians|People of the Russia] wake up plax!
                            required:
                                - type
                                - content
    """
    results = []
    for msg in get_messages():
        results.append({'type': msg.msg_type,
                        'content': msg.msg_content})
    return results




from __future__ import annotations

import json
import random
import re
import typing as tp

import requests

REPL_RE = re.compile(r'\[(.*?)\]')


def get_messages(part_type: tp.Optional[str] = None) -> tp.List[MessagePiece]:
    with open('/app/resources/messages-short.json', 'r') as f_in:
        data = json.load(f_in)
    if part_type is None:
        return [MessagePiece.from_json(y) for y in data]
    else:
        return [MessagePiece.from_json(y) for y in data if y['type'] == part_type]


messages = []


def get_messages_backend(part_type: tp.Optional[str] = None) -> tp.List[MessagePiece]:
    global messages

    if not messages:
        msgs = requests.get('https://yandex.henrietta.com.pl/v1/view-messages')
        msgs.raise_for_status()
        messages = [MessagePiece(msg['type'], msg['content']) for msg in msgs.json()]

    if not part_type:
        return messages
    else:
        return [msg for msg in messages if msg.msg_type == part_type]


class MessagePiece:
    def __init__(self, msg_type: str, msg_content: str):
        self.msg_type = msg_type
        self.msg_content = msg_content

    @staticmethod
    def from_json(y: dict) -> MessagePiece:
        return MessagePiece(y['type'], y['content'])

    def __str__(self):
        def replacer(match):
            return random.choice(match.group(1).split('|'))

        return re.sub(REPL_RE, replacer, self.msg_content)



import random

from .definitions import get_messages, get_messages_backend


def get_msg_using_provider(provider):
    return str(random.choice(provider('1-A')))


def generate_message() -> str:
    return get_msg_using_provider(get_messages)


def generate_message_client() -> str:
    return get_msg_using_provider(get_messages_backend)



import os

import flask
import flask_json
import yaml
from flasgger import Swagger
from flask_cors import CORS

from flask_satella_metrics import SatellaMetricsMiddleware
from satella.instrumentation.metrics import getMetric
from flask_satella_metrics.prometheus_exporter import PrometheusExporter

from flask_requests_logging import FlaskRequestsLogging

__version__ = '1.0'

app = flask.Flask(__name__)

app.config['JSON_ADD_STATUS'] = False
app.config['DEBUG'] = os.environ.get('DEBUG', '1') == '1'
app.config['TESTING'] = os.environ.get('CI', '0') == '1'

flask_json.FlaskJSON(app)
CORS(app, supports_credentials=True)
FlaskRequestsLogging(app)

with open('/app/app-template.yml', 'r') as f_in:
    template = yaml.load(f_in, Loader=yaml.Loader)

template['info']['version'] = __version__
app.config['SWAGGER'] = {'openapi': '3.0.2'}
Swagger(app, template=template)


@app.route('/')
def redirect_to_swagger():
    return flask.redirect('/apidocs')


SatellaMetricsMiddleware(app, summary_metric=getMetric('yandex.api.call_time.summary', 'summary',
                                                       quantiles=[0.5, 0.95, 0.99]),
                         histogram_metric=getMetric('yandex.api.call_time.histogram', 'histogram'),
                         response_codes_metric=getMetric('yandex.api.response_codes', 'counter'))

app.register_blueprint(PrometheusExporter({'service_name': 'yandex'}))
